#-------------------------------------------------------------------------------
# Name:        quadratanalysis.py
# Purpose:
# Author:
# Created:     23/06/2011
# Copyright:   (c) esri
# ArcGIS Version:   10
# Python Version:   2.6
#-------------------------------------------------------------------------------
import numpy
import locale
import arcpy
import Stats
import SSUtilities


def poisson_chi_squared(numRows, numCols, frequencyList):
    """
    Calculate the distribution of points within the set of quadrats using the
    chisquare statistic.
    """
    freq_array = numpy.array(frequencyList)
    sumPoints = freq_array.sum()
    sumCells = (numRows * 1.0) * numCols
    nBar = sumPoints / sumCells
    chiValue = (((freq_array - nBar)**2.0) / nBar ).sum()
    dof = int(sumCells - 1)
    chiProb = Stats.chiProb(chiValue, dof, type = 1)

    arcpy.AddMessage("P-Value: {0}".format(chiProb))
    arcpy.SetParameter(3, chiProb)
# End poisson_chi_squared function


def main(input_points, rows, columns, *args):
    '''Main function to perform quadrat analysis.'''

    try:
        # Get the extent of the input point feature class
        extent = arcpy.Describe(input_points).extent
        # Create the quadrats using the extent of point features
        quadrats = arcpy.CreateFishnet_management(r'in_memory\fishnet',
                                    '{0} {1}'.format(extent.XMin, extent.YMin),
                                    '{0} {1}'.format(extent.XMin, extent.YMax),
                                    0, 0, rows, columns,
                                    '{0} {1}'.format(extent.XMax, extent.YMax), 'NO_LABELS',
                                    extent, 'POLYGON')
    
        # Determine the frequency of points within each quadrat
        identity = arcpy.Identity_analysis(input_points, quadrats, r'in_memory\points_identity', 'ONLY_FID')
        point_frequency = arcpy.Frequency_analysis(identity, r'in_memory\point_frq', 'FID_fishnet')
    
        # Create a list of the point frequencies
        srows = arcpy.SearchCursor(point_frequency, '', '', 'FREQUENCY')
        frequency_list = [srow.getValue('FREQUENCY') for srow in srows]
    
        # Perform chi-squared statistic to detemine the spatial pattern of the points in the set of quadrats
        poisson_chi_squared(int(rows), int(columns), frequency_list)

    except arcpy.ExecuteError:
        arcpy.AddError(arcpy.GetMessages(2))
    except Exception as e:
        arcpy.AddError(e.args[0])
# End main function

# Makes this file is usable as a script,
# script tool, or as an importable module
if __name__ == '__main__':
    argv = tuple(arcpy.GetParameterAsText(i)
                 for i in range(arcpy.GetArgumentCount()))
    main(*argv)